#include "TArtRecoRIPS.h" 
#include "TArtRIPS.h"
#include "TArtFocalPlane.h"
#include "TArtBigRIPSParameters.h"
#include "TArtFocalPlanePara.h"
#include "TArtCore.h" 
#include "TArtStoreManager.h"

#include <TROOT.h>
#include <TMatrixD.h>
#include <TClonesArray.h>

#include <fstream>

//__________________________________________________________
TArtRecoRIPS::TArtRecoRIPS()
  :fReconstructed(false)
{
  TArtCore::Info(__FILE__,"Creating the RIPS objects...");
  sman = TArtStoreManager::Instance();
  setup = (TArtBigRIPSParameters*)sman->FindParameters("BigRIPSParameters");
  if(!setup){
    TArtCore::Info(__FILE__,"Can not find parameter handler: BigRIPSParameters");
    throw;
  }

  fRIPSArray = new TClonesArray("TArtRIPS",10);
  fRIPSArray->SetName("BigRIPSRIPS");
  fRIPSArray->SetOwner();
  sman->AddDataContainer(fRIPSArray);
}

//__________________________________________________________
TArtRecoRIPS::~TArtRecoRIPS()  {
  ClearData();
}

//__________________________________________________________
TArtRIPS * TArtRecoRIPS::DefineNewRIPS(Int_t ufpl, Int_t dfpl, char *file, Double_t brho)   {

  char name[64];
  Int_t nrips=GetNumRIPS();
  new ((*fRIPSArray)[nrips]) TArtRIPS();
  TArtRIPS * rips = (TArtRIPS *)fRIPSArray->At(nrips);
  TMatrixD * matrix = (TMatrixD *)rips->GetMatrix();

  char buffer[256];
  std::ifstream matfin(file); 
  matfin.getline(buffer,256);
  matrix->ResizeTo(6,5); 
  double val[6];  
  for(int i=0;i<6;i++){
    matfin>>val[0]>>val[1]>>val[2]>>val[3]>>val[4]>>val[5];
    for(int j=0;j<5;j++)
      (*matrix)(i,j) = val[j];
  }
  matfin.close();

  sprintf(name,"RIPS%dto%d",ufpl,dfpl);
  TArtCore::Info(__FILE__,"define %s",name);
  rips->SetDetectorName(name);
  rips->SetUpstreamFpl(ufpl);
  rips->SetDownstreamFpl(dfpl);
  rips->SetBrho(brho);
  TArtFocalPlanePara * ufpl_para = setup->GetFocalPlanePara(ufpl);
  TArtFocalPlanePara * dfpl_para = setup->GetFocalPlanePara(dfpl);
  if(NULL == ufpl_para)
    TArtCore::Info(__FILE__,"can not find fol object for defining, up:%d",ufpl);
  if(NULL == dfpl_para)
    TArtCore::Info(__FILE__,"can not find fol object for defining, down:%d ",dfpl);
  rips->SetLength( dfpl_para->GetZpos() - ufpl_para->GetZpos() );

  return rips;

}

//__________________________________________________________
void TArtRecoRIPS::ClearData()   {
  for(int i=0;i<GetNumRIPS();i++)
    fRIPSArray->At(i)->Clear();
  fReconstructed = false;
  return;
}

//__________________________________________________________
void TArtRecoRIPS::ReconstructData()   { // call after the raw data are loaded

  TClonesArray * fpl_array = (TClonesArray *)sman->FindDataContainer("BigRIPSFocalPlane");
  Int_t num_fpl = fpl_array->GetEntries();

  //on the assumption of linear track, the track is reconstructed analytically.
  //X = a + b * Z;

  for(int i=0;i<GetNumRIPS();i++){

    TArtRIPS *rips = (TArtRIPS *)fRIPSArray->At(i);
    TArtFocalPlane * upstream_fpl = NULL;
    TArtFocalPlane * downstream_fpl = NULL;

    /////////////////////////////////////////////////////////////////////
    // find fpl objects
    bool FindAllObjects = true;

    for(int j=0;j<num_fpl;j++){
      TArtFocalPlane * tmp_fpl = (TArtFocalPlane *)fpl_array->At(j);
      if(tmp_fpl->GetFpl() == rips->GetUpstreamFpl()){
	upstream_fpl = tmp_fpl;
      }
      if(tmp_fpl->GetFpl() == rips->GetDownstreamFpl()){
	downstream_fpl = tmp_fpl;
      }
    }

    if(NULL == upstream_fpl){
      TArtCore::Info(__FILE__,"no upstream fpl: %d",rips->GetUpstreamFpl());
      FindAllObjects = false;
    }
    if(NULL == downstream_fpl){
      TArtCore::Info(__FILE__,"no downstream fpl: %d",rips->GetDownstreamFpl());
      FindAllObjects = false;
    }

    if(!FindAllObjects) continue;
    /////////////////////////////////////////////////////////////////////
    // start to reconstruct 

    TMatrixD *matrix = (TMatrixD *)rips->GetMatrix();
    if(NULL==matrix){
      TArtCore::Info(__FILE__,"matrix is no defined!: %d - %d",rips->GetUpstreamFpl(), rips->GetDownstreamFpl());
      continue;
    }
    TMatrixD mat1, mat2;
    mat1.ResizeTo(2,1);
    mat1(0,0) = (*matrix)(0,0); mat1(1,0) = (*matrix)(0,1);
    //mat1.Print();
    mat2.ResizeTo(2,2);
    mat2(0,0) = (*matrix)(1,0); mat2(0,1) = (*matrix)(5,0);
    mat2(1,0) = (*matrix)(1,1); mat2(1,1) = (*matrix)(5,1);
    //mat2.Print();
    mat2.Invert();

    TVectorD * fvec = (TVectorD *)upstream_fpl->GetOptVector();
    TVectorD * bvec = (TVectorD *)downstream_fpl->GetOptVector();

    if(!fvec || !bvec)  // if optical vector can not be taken
      return;

    TMatrixD xvec(2,1);
    xvec(0,0) = (*bvec)(0);
    xvec(1,0) = (*bvec)(1);

    TMatrixD rvec = mat2 * (xvec - (*fvec)(0) * mat1);
    Double_t angle = rvec(0,0); // change to mrad
    Double_t delta = rvec(1,0);
    // simple_delta = (*bvec)(0) / 33.; // only for fpl-5

    rips->SetDelta(delta);
    rips->SetAngle(angle);

  } 

  fReconstructed = true;
  return;
}

//__________________________________________________________
TArtRIPS * TArtRecoRIPS::GetRIPS(int i) {
  return (TArtRIPS *)fRIPSArray->At(i);
}
//__________________________________________________________
Int_t TArtRecoRIPS::GetNumRIPS() {
  return fRIPSArray->GetEntries();
}
//__________________________________________________________
TArtRIPS * TArtRecoRIPS::FindRIPS(Int_t u_fpl, Int_t d_fpl){
  for(int i=0;i<GetNumRIPS();i++)
    if(u_fpl == ((TArtRIPS*)fRIPSArray->At(i))->GetUpstreamFpl()
       && d_fpl == ((TArtRIPS*)fRIPSArray->At(i))->GetDownstreamFpl())
      return (TArtRIPS*)fRIPSArray->At(i);
  return NULL;
}
