#include "TArtCalibSiStopper.h" 
#include "TArtSiStopper.h"
#include "TArtSiStopperPara.h"
#include "TArtEURICAParameters.h"
#include "TArtRawEventObject.h"
#include "TArtCore.h" 
#include "TArtStoreManager.h"

#include <TROOT.h>
#include <TMath.h>
#include <TClonesArray.h>

//__________________________________________________________
TArtCalibSiStopper::TArtCalibSiStopper()
  :fReconstructed(false), fDataLoaded(false)
{
  TArtCore::Info(__FILE__,"Creating the SiStopper detector objects...");

  sman = TArtStoreManager::Instance();
  setup = (TArtEURICAParameters*)sman->FindParameters("EURICAParameters");
  if(!setup){
    TArtCore::Info(__FILE__,"Can not find parameter handler: EURICAParameters");
    throw;
  }

  fSiStopperArray = new TClonesArray("TArtSiStopper",10);
  fSiStopperArray->SetName("EURICASiStopper");
  fSiStopperArray->SetOwner();
  sman->AddDataContainer(fSiStopperArray);

}

//__________________________________________________________
TArtCalibSiStopper::~TArtCalibSiStopper()  {
  ClearData();
}

//__________________________________________________________
void TArtCalibSiStopper::LoadData()   {

  TArtRawEventObject *fEvent = (TArtRawEventObject *)sman->FindDataContainer("RawEvent");

  ClearData();
  for(int i=0;i<fEvent->GetNumSeg();i++){
    TArtRawSegmentObject *seg = fEvent->GetSegment(i);
    int device   = seg->GetDevice();
    int fpl      = seg->GetFP();
    int detector = seg->GetDetector();
    if((ZeroDegree == device) && (11==fpl) && (BETAA==detector || BETAT==detector || HIA==detector)) LoadData(seg);
  }
  return;

}

//__________________________________________________________
void TArtCalibSiStopper::LoadData(TArtRawSegmentObject *seg)   {

  int device   = seg->GetDevice();
  int fpl      = seg->GetFP();
  int detector = seg->GetDetector();
  for(int j=0;j<seg->GetNumData();j++){
    TArtRawDataObject *d = seg->GetData(j);
    int geo = d->GetGeo(); 
    int ch = d->GetCh(); 
    unsigned int val = d->GetVal();
    TArtRIDFMap mm(fpl,detector,geo,ch);
    TArtSiStopperPara *para = (TArtSiStopperPara*)setup->GetSiStopperPara(&mm);
    if(NULL == para){
      TArtCore::Info(__FILE__,"Could not find TArtSiStopperPara...: Dev:%d, %s", device, mm.GetMapInfo());
      continue;
    }

    int id = para->GetID();
    int nsi = fSiStopperArray->GetEntries();
    TArtCore::Debug(__FILE__,"Entries in SiStopperArray: %d, SiStopper-ID:%d", nsi, id);
    TArtSiStopper * si = FindSiStopper(id);

    if(NULL==si){
      new ((*fSiStopperArray)[nsi]) TArtSiStopper();
      si = (TArtSiStopper *)fSiStopperArray->At(nsi);
      si->SetID(id);
      fSiStopperParaArray.push_back(para);
    }

    // set raw data
    // do not need to scan channel because of only 1 channel for 1 si.
    if(BETAA == detector)
      si->SetRawADC(val);
    if(HIA == detector)
      si->SetRawHIADC(val);
    if(BETAT == detector)
      si->SetRawTDC(val);

  }

  fDataLoaded = true;
  return;
}

//__________________________________________________________
void TArtCalibSiStopper::ClearData()   {
  fSiStopperArray->Delete();
  fSiStopperParaArray.clear();
  fDataLoaded = false;
  fReconstructed = false;
  return;
}

//__________________________________________________________
void TArtCalibSiStopper::ReconstructData()   { // call after the raw data are loaded

  if(!fDataLoaded) LoadData();

  for(int i=0;i<GetNumSiStopper();i++){
    TArtSiStopper *si = (TArtSiStopper*)fSiStopperArray->At(i);
    TArtSiStopperPara *para = fSiStopperParaArray[i];

    unsigned int adc = si->GetRawADC();
    unsigned int hiadc = si->GetRawHIADC();
    unsigned int tdc = si->GetRawTDC();

    // copy some information from para to data container
    si->SetID(para->GetID());
    si->SetDetectorName(*(para->GetDetectorName()));
    si->SetLayer(para->GetLayer());

    if(tdc>0 && tdc<100000){ 
      Double_t fTime = tdc * para->GetTCal();
      si->SetTime(fTime);
    }
    if(adc>0 && adc<100000){ 
      Double_t fEnergy = (adc - para->GetQPed()) * para->GetQCal();
      si->SetEnergy(fEnergy);
    }
    if(hiadc>0 && hiadc<100000){ 
      Double_t fHIEnergy = (hiadc - para->GetHIQPed()) * para->GetHIQCal();
      si->SetHIEnergy(fHIEnergy);
    }

  }

  fReconstructed = true;
  return;
}

//__________________________________________________________
TArtSiStopper * TArtCalibSiStopper::GetSiStopper(int i) {
  return (TArtSiStopper *)fSiStopperArray->At(i);
}
//__________________________________________________________
TArtSiStopperPara * TArtCalibSiStopper::GetSiStopperPara(int i) {
  return (TArtSiStopperPara *)fSiStopperParaArray[i];
}
//__________________________________________________________
Int_t TArtCalibSiStopper::GetNumSiStopper() {
  return fSiStopperArray->GetEntries();
}
//__________________________________________________________
TArtSiStopper * TArtCalibSiStopper::FindSiStopper(Int_t id){
  for(int i=0;i<GetNumSiStopper();i++)
    if(id == ((TArtSiStopper*)fSiStopperArray->At(i))->GetID())
      return (TArtSiStopper*)fSiStopperArray->At(i);
  return NULL;
}

