
#include "TArtCalibNEBULA.hh" 

#include "TArtRawEventObject.h"
#include "TArtCore.h" 
#include "TArtStoreManager.h"
#include "TArtMapper.h"
#include "segidlist.h"

#include "TArtSAMURAIParameters.hh"
#include "TArtNEBULAPla.hh"
#include "TArtNEBULAPlaPara.hh"

#include <TClonesArray.h>
#include <iostream>
#include <cmath>

//__________________________________________________________
TArtCalibNEBULA::TArtCalibNEBULA() : fReconstructed(false), fSAMURAIParameters(0)
{
  TArtCore::Info(__FILE__,"Creating the NEBULA detector objects...");

  fStoreManager = TArtStoreManager::Instance();
  fNEBULAPlaArray = new TArtNEBULAPlaArray("TArtNEBULAPla", 300);
  fNEBULAPlaArray->SetOwner();
  fNEBULAPlaArray->SetName("NEBULAPla");
  fNEBULAPlaParaArray = new TArtNEBULAPlaParaArray("TArtNEBULAPlaPara", 300);
  fNEBULAPlaParaArray->SetOwner();
  fNEBULAPlaParaArray->SetName("NEBULAPlaPara");
  fStoreManager->AddDataContainer(fNEBULAPlaArray);
  fNEBULAPlaParaArray = new TArtNEBULAPlaParaArray;
  fSAMURAIParameters = (const TArtSAMURAIParameters*)fStoreManager->FindParameters("SAMURAIParameters");
  if(!fSAMURAIParameters){
    TArtCore::Info(__FILE__,"Could not find SAMURAIParameters.");
    throw;
  }
}

//__________________________________________________________
TArtCalibNEBULA::~TArtCalibNEBULA()
{
  delete fNEBULAPlaParaArray;
  delete fNEBULAPlaArray;
}

//__________________________________________________________
void TArtCalibNEBULA::LoadRawData()
{
  TArtRawEventObject* fEvent = (TArtRawEventObject*)fStoreManager->FindDataContainer("RawEvent");
  ClearData();

  for(int i=0; i<fEvent->GetNumSeg(); ++i){
    TArtRawSegmentObject* seg = fEvent->GetSegment(i);
    int device = seg->GetDevice();
    int fpl = seg->GetFP();
    int detector = seg->GetDetector();
    if(SAMURAI != device) continue;
    if(detector != 41 && detector != 42) continue; // analysis only tdc and qdc

    for(int j=0; j<seg->GetNumData(); ++j){
      TArtRawDataObject* d = seg->GetData(j);
      int geo = d->GetGeo(); 
      int ch = d->GetCh(); 
      unsigned int val = d->GetVal();
      TArtRIDFMap mm(fpl,detector,geo,ch);
      const TArtNEBULAPlaPara* parau = fSAMURAIParameters->FindNEBULAPlaParaU(mm);
      const TArtNEBULAPlaPara* parad = fSAMURAIParameters->FindNEBULAPlaParaD(mm);
      const TArtNEBULAPlaPara* para;
      if(parau && !parad){
	para = parau;
      }else if(!parau && parad){
	para = parad;
      }else if(parau && parad){
	TArtCore::Info(__FILE__,"TArtNEBULAPara...: Dev:%d, %s found in U and D",
		       device, mm.GetMapInfo());
	continue;
      }else{
	TArtCore::Info(__FILE__,"Could not find TArtNEBULAPlaPara...: Dev:%d, %s",
		       device, mm.GetMapInfo());
	continue;
      }
      if(para->GetIgnore()){
	TArtCore::Debug(__FILE__,"Ignore TArtNEBULAPlaPara...: Dev:%d, %s", device, mm.GetMapInfo());
	continue;
      }

      if(FindNEBULAPlaPara(para->GetID())){
	int npara = fNEBULAPlaParaArray->GetEntries();
	new ((*fNEBULAPlaParaArray)[npara]) TArtNEBULAPlaPara(*para);

	fIDNPlaParaMap.insert(std::pair<int, int>(para->GetID(), npara));
      }
      TArtNEBULAPla* pla = FindNEBULAPla(para->GetID());
      if(!pla){
	int npla = fNEBULAPlaArray->GetEntries();
	pla = new ((*fNEBULAPlaArray)[npla]) TArtNEBULAPla();
	pla->SetID(para->GetID());
	pla->SetFPl(para->GetFPl());
	pla->SetDetectorName(para->GetDetectorName());
	pla->SetLayer(para->GetLayer());
	pla->SetSubLayer(para->GetSubLayer());
	pla->SetDetPos(para->GetDetPos());

	fIDNPlaMap.insert(std::pair<int, int>(para->GetID(), npla));
      }

      // if(detector == NEBULAT)
      if(detector == 41){
	if(parau){
	  pla->SetTURaw(val);
	}else{
	  pla->SetTDRaw(val);
	}
      }
      // if(detector == NEBULAQ)
      if(detector == 42){
	if(parau){
	  pla->SetQURaw(val);
	}else{
	  pla->SetQDRaw(val);
	}
      }
    }
  }
}

//__________________________________________________________
void TArtCalibNEBULA::ReconstructData()
{
  //共通のoffset。どう値を取ってくるかは後で考える(とりあえず手打ち？)。
  Double_t posxoff = 0;
  Double_t posyoff = 0;
  Double_t poszoff = 0;

  for(int i=0;i<GetNumNEBULAPla();++i){
    TArtNEBULAPla* pla = GetNEBULAPla(i);
    Int_t id = pla->GetID();
    const TArtNEBULAPlaPara* para = fSAMURAIParameters->FindNEBULAPlaPara(id);
    if(!para){
      TArtCore::Info(__FILE__,"cannot find para %d U", id); // programming problem
      continue;
    }
    int turaw = pla->GetTURaw();
    int tdraw = pla->GetTDRaw();
    int quraw = pla->GetQURaw();
    int qdraw = pla->GetQDRaw();

    int ihitmin = 0;
    if(turaw>0 && turaw<=4096) ++ihitmin;
    if(tdraw>0 && tdraw<=4096) ++ihitmin;      
    if(quraw>0 && quraw<=4096) ++ihitmin;
    if(qdraw>0 && qdraw<=4096) ++ihitmin;
    pla->SetIHitMin(ihitmin);

    Double_t quped = quraw - para->GetQUPed();
    Double_t qdped = qdraw - para->GetQDPed();
    Double_t qucal = quped * para->GetQUCal();
    Double_t qdcal = qdped * para->GetQDCal();
    Double_t qaveped = sqrt(quped*qdped);
    Double_t qavecal = sqrt(qucal*qdcal);
    Double_t logqped = log(quped/qdped);
    Double_t logqcal = log(qucal/qdcal);

    Double_t tucal = turaw * para->GetTUCal() + para->GetTUOff();
    Double_t tdcal = tdraw * para->GetTDCal() + para->GetTDOff();
    Double_t tuslw = tucal - para->GetTUSlw()/sqrt(quped);
    Double_t tdslw = tdcal - para->GetTDSlw()/sqrt(qdped);
    Double_t dtraw = turaw - tdraw;
    Double_t dtcal = tucal - tdcal;
    Double_t dtslw = tuslw - tdslw;
    Double_t taveraw = (turaw + tdraw)/2.;
    Double_t tavecal = (tucal + tdcal)/2 + para->GetTAveOff();
    Double_t taveslw = (tuslw + tdslw)/2 + para->GetTAveOff();

    Double_t poscal = dtcal * para->GetDTCal() + para->GetDTOff();
    Double_t posslw = dtslw * para->GetDTCal() + para->GetDTOff();

    Double_t pos[3];
    pos[0] =          para->GetDetPos(0) + posxoff;
    pos[1] = posslw + para->GetDetPos(1) + posyoff; 
    pos[2] =          para->GetDetPos(2) + poszoff;

    pla->SetQUPed(quped);
    pla->SetQDPed(qdped);
    pla->SetQUCal(qucal);
    pla->SetQDCal(qdcal);
    pla->SetQAvePed(qaveped);
    pla->SetQAveCal(qavecal);
    pla->SetLogQPed(logqped);
    pla->SetLogQCal(logqcal);

    pla->SetTUCal(tucal);
    pla->SetTDCal(tdcal);
    pla->SetTUSlw(tuslw);
    pla->SetTDSlw(tdslw);
    pla->SetDTRaw(dtraw);
    pla->SetDTCal(dtcal);
    pla->SetDTSlw(dtslw);    
    pla->SetTAveRaw(taveraw);
    pla->SetTAveCal(tavecal);
    pla->SetTAveSlw(taveslw);    

    pla->SetPosCal(poscal);
    pla->SetPosSlw(posslw);
    pla->SetPos(pos);

    pla->SetDataState(1);
  }
  
  fReconstructed = true;
}

//__________________________________________________________
Int_t TArtCalibNEBULA::GetNumNEBULAPla() const
{
  return fNEBULAPlaArray->GetEntries();
}

//__________________________________________________________
TArtNEBULAPla* TArtCalibNEBULA::GetNEBULAPla(int i) const
{
  return (TArtNEBULAPla*)fNEBULAPlaArray->At(i);
}

//__________________________________________________________
TArtNEBULAPla* TArtCalibNEBULA::FindNEBULAPla(Int_t id) const
{
  std::map<int,int>::const_iterator it = fIDNPlaMap.find(id);
  if(it != fIDNPlaMap.end()) return GetNEBULAPla(it->second);
  else return 0;
}

//__________________________________________________________
TArtNEBULAPlaArray* TArtCalibNEBULA::GetNEBULAPlaArray() const
{
  return fNEBULAPlaArray;
}

//__________________________________________________________
Int_t TArtCalibNEBULA::GetNumNEBULAPlaPara() const
{
  return fNEBULAPlaParaArray->GetEntries();
}

//__________________________________________________________
TArtNEBULAPlaPara* TArtCalibNEBULA::GetNEBULAPlaPara(int i) const
{
  return (TArtNEBULAPlaPara*)fNEBULAPlaParaArray->At(i);
}

//__________________________________________________________
TArtNEBULAPlaPara* TArtCalibNEBULA::FindNEBULAPlaPara(int id) const
{
  std::map<int,int>::const_iterator it = fIDNPlaParaMap.find(id);
  if(it != fIDNPlaParaMap.end()) return GetNEBULAPlaPara(it->second);
  else return 0;
}

//__________________________________________________________
TArtNEBULAPlaParaArray* TArtCalibNEBULA::GetNEBULAPlaParaArray() const
{
  return fNEBULAPlaParaArray;
}

//__________________________________________________________
void TArtCalibNEBULA::ClearData()
{
  fNEBULAPlaArray->Clear();
  fNEBULAPlaParaArray->Clear();
  fIDNPlaMap.clear();
  fIDNPlaParaMap.clear();
  fReconstructed = false;
}

